@@ -20,8 +20,11 @@
  *
  */
 
-#include "gmime-content-type.h"
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include "gmime-content-type.h"
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -49,10 +52,15 @@ g_mime_content_type_new (const gchar *type, const gchar *subtype)
 	} else {
 		if (type && *type) {
 			mime_type->type = g_strdup (type);
-			if (!g_strcasecmp (type, "text"))
+			if (!g_strcasecmp (type, "text")) {
 				mime_type->subtype = g_strdup ("plain");
-			else
+			} else if (!g_strcasecmp (type, "multipart")) {
+				mime_type->subtype = g_strdup ("mixed");
+			} else {
+				g_free (mime_type->type);
+				mime_type->type = g_strdup ("application");
 				mime_type->subtype = g_strdup ("octet-stream");
+			}
 		} else {
 			mime_type->type = g_strdup ("application");
 			mime_type->subtype = g_strdup ("octet-stream");
@@ -102,7 +110,7 @@ g_mime_content_type_new_from_string (const gchar *string)
 	
 	while (*eptr == ';') {
 		/* looks like we've got some parameters */
-		guchar *name, *value, *ch;
+		guchar *name, *value;
 		
 		while (*eptr && (*eptr == ';' || isspace (*eptr)))
 			eptr++;
@@ -287,7 +295,9 @@ g_mime_content_type_get_parameter (const GMimeContentType *mime_type, const gcha
 	GMimeParam *param;
 	
 	g_return_val_if_fail (mime_type != NULL, NULL);
-	g_return_val_if_fail (mime_type->param_hash != NULL, NULL);
+	
+	if (!mime_type->param_hash)
+		return NULL;
 	
 	param = g_hash_table_lookup (mime_type->param_hash, attribute);
 	
